﻿/**
********************************************************************************
Ortus DataBoss
Copyright 2012 Ortus Solutions, Corp
www.ortussolutions.com
********************************************************************************
* A user entity
*/
component persistent="true" table="user" displayName="User" db_sortBy="lastName"{

	// Properties
	property name="userID" 		fieldtype="id" generator="native" setter="false";
	property name="firstName"	length="100" notnull="true" db_labelText="First Name" db_placeholder="John";
	property name="lastName"	length="100" notnull="true" db_labelText="Last Name" db_placeholder="Doe";
	property name="email"		length="255" notnull="true" db_html="email" db_labelText="Email Address";
	property name="username"	length="100" notnull="true" unique="true" db_labelText="Username" db_display="false";
	property name="password"	length="100" notnull="true" db_labelText="Password" db_html="password" db_display="false";
	property name="isActive" 	ormtype="boolean"   notnull="true" default="false" dbdefault="0" db_labelText="Active";
	property name="description" ormtype="text" notnull="false" default="" db_labelText="Description" db_html="richtext" db_display="false";
	property name="lastLogin" 	ormtype="timestamp" notnull="false" db_labelText="Last Login" db_dateformat="mm/dd/yyyy" db_timeformat="full";
	property name="createdDate" ormtype="timestamp" notnull="true" update="false" db_labelText="Created Date"  db_dateformat="mm/dd/yyyy";
	property name="updatedDate" ormtype="timestamp" sqltype="timestamp" notnull="true" insert="false" update="false" db_labelText="Updated Date" db_dateformat="mm/dd/yyyy";
	
	// M20 -> Role
	property name="role" notnull="true" fieldtype="many-to-one" cfc="Role" fkcolumn="FK_roleID" lazy="true" db_labelText="Role" db_displayColumns="Role";

	// M2M -> A-la-carte Author Permissions
	property name="permissions" singularName="permission" fieldtype="many-to-many" type="array" lazy="extra"
			 cfc="Permission" cascade="all"
			 fkcolumn="FK_userID" linktable="cb_authorPermissions" inversejoincolumn="FK_permissionID" orderby="permission";
	
	// Validation Constraints
	this.constraints ={
		"firstName" = {required=true, size="1..100"},
		"lastName" 	= {required=true, size="1..100"},
		"email" 	= {required=true, size="1..255", type="email"},
		"username" 	= {required=true, size="1..100", unique="true"},
		"password"	= {required=true, size="1..100"},
		"isActive"	= {required=true, type="boolean"}
	};

	/* ----------------------------------------- ORM EVENTS -----------------------------------------  */

	/*
	* In built event handler method, which is called if you set ormsettings.eventhandler = true in Application.cfc
	*/
	public void function preInsert(){
		setCreatedDate( now() );
	}

	/* ----------------------------------------- PUBLIC -----------------------------------------  */

	/**
	* Constructor
	*/
	function init(){
		return this;
	}

	/**
	* Get formatted lastLogin
	*/
	string function getDisplayLastLogin(){
		var lastLogin = getLastLogin();

		if(  NOT isNull(lastLogin) ){
			return dateFormat( lastLogin, "mm/dd/yyy" ) & " " & timeFormat(lastLogin, "hh:mm:ss tt");
		}

		return "Never";
	}

	/**
	* Get formatted createdDate
	*/
	string function getDisplayCreatedDate(){
		var createdDate = getCreatedDate();
		if( isNull( createdDate ) ){ return ""; }
		return dateFormat( createdDate, "mm/dd/yyy" ) & " " & timeFormat(createdDate, "hh:mm:ss tt");
	}

	/**
	* Retrieve full name
	*/
	string function getName(){
		return getFirstName() & " " & getLastName();
	}

	/**
	* is loaded?
	*/
	boolean function isLoaded(){
		return len( getUserID() );
	}
	
}